;+
; @file_comments	perform an sql query on MPA databases and return csv format answer
;       
; @examples	result = query_sql(query, /login, server='http://gavows2.xray.mpe.mpg.de:8080/lgalaxies/DBQueryStream' )
;       
; @param query - string containing an sql query to be executed on the database
;
; @param server - optional string containing an alternative server location
;       
; @returns	result - a csv formatted array, one line per element, containing the results of the query
;
; @history - Originally created Jan 2006 by Ben Panter. Modifications and improvements suggested by Viv Wild, Manfred Kitzbichler, Eric Hayashi and Jeremy Blaizot. All SQL servers maintained by Gerard Lemson
;
;       
; @keyword login - force login (login is handled once per session by a widget, can force re-login with this
;
; @keyword server - change SQL server. This can be changed permanently below, between two lines of stars
;
; @file_comments
;
;  	put your query in a string:
;
;	IDL> query = 'select * from agebin'
;
;	then run the search:
;
;	IDL> result = query_sql(query)
;
;	(a little widget will pop up and ask for your username and login.
;	 At the moment there are no stars for password, but I'm working on it!)
;
;	IDL> print, transpose(result)
;
;	result is a csv formated array of strings, one per line of result, with two comments at the top and a list of other things. Later versions will be able to return a structure, but not just yet!
;	You'll need to login for the first query of each session, but shouldn't after that. If you have the wrong login details and need to replace them, simply type /login as a keyword when you call query_sql:
;
;	IDL> result = query_sql(query, /login)
;
;   If you're using the Millenium SQL database, you should use a seperate server. Something like:
;  result = query_sql(query, /login, server='http://gavows2.xray.mpe.mpg.de:8080/lgalaxies/DBQueryStream' )
;
; @history:  created 12/1/06 Ben Panter
;       
; @requires:	username and password for Gerard Lemson's gateway to the MPA SDSS/Millenium database
;       
;-

PRO sql_login_event, event

;event handler for widget

eventName = Tag_Names(event, /structure_name)
if eventNAme ne 'WIDGET_BUTTON' then return

widget_control, event.top, get_uvalue=info, /no_copy
widget_control, event.id, get_value=buttonvalue

case buttonvalue of 
	'Cancel' : Widget_control, event.top, /destroy
	'  OK  ' : begin
	Widget_control, info.usernameID, get_value=username
	widget_control, info.passwordID, get_value=password
	(*info.ptr).username=username
	(*info.ptr).password=password
	(*info.ptr).cancel=0
	Widget_control, event.top, /Destroy
	END
ENDCASE

END

FUNCTION sql_login

;handles the login via a widget

tlb= widget_base(column=1, /base_align_center, title='You must login to perform an SQL query')
subbase=widget_base(tlb, column=1, Frame=1)
filebase= widget_base(subbase, row=1)

;data fields
usernameID= cw_field(filebase, title='Username: ', value=username)
passwordID= cw_field(filebase, title='Password: ', value=password)

;now make the buttons
butbase = widget_base(tlb, Row=1)
ok = widget_button(butbase, value='  OK  ')
cancel = widget_button(butbase, value='Cancel')

	
screensize=get_screen_size()
geom= widget_info(tlb, /geometry)
widget_control, tlb, $
	Xoffset = (screensize[0] / 2) - (geom.Scr_Xsize / 2), $
	Yoffset = (screensize[1] / 2) - (geom.Scr_Ysize / 2)
		
centerTLB, tlb

Widget_control, tlb, /Realize

ptr=Ptr_New({Username:'', Password:'', Cancel:1})

info={usernameID:usernameID, passwordID:passwordID, ptr:ptr}

Widget_control, tlb, set_uvalue=info, /no_copy

xmanager, 'sql_login', tlb

fileinfo=*ptr
Ptr_free, ptr

cancel=fileinfo.cancel

return, fileinfo

END

FUNCTION query_sql, query, login=login, server=server, silent=silent

;store details for a session in a common block

common sql_details, user, passwd

if n_elements(user) eq 0 or keyword_set(login) then begin
	result=sql_login()
	if result.cancel eq 1 then return, '# User must login to access SQL data'
	user=result.username
	passwd=result.password
	end

;construct wget command

;**********************************************************************************
;Feel free to change this DEFAULT server if you get bored of typing the server keyword:

default_server='http://gavows2.xray.mpe.mpg.de:8080/sdssdr3/DBQueryStream'

;other servers:
;server='http://gavows2.xray.mpe.mpg.de:8080/lgalaxies/DBQueryStream' this is probably what you want if you wish to go direct to lgalaxies
;server='http://gavows2.xray.mpe.mpg.de:8080/sdssdr3/DBQueryStream' this is the sdss server

if keyword_set(server) then server=server else server=default_server
;**********************************************************************************

;compress a multi-line query to a single statement:

query=strjoin(query, ' ')

if keyword_set(silent) then quiet_str=' -q ' else quiet_str=''

command='wget -nv '+quiet_str+'--http-user='+user+' --http-passwd='+passwd+' -O - "'+server+'?SQL='+query+'"

if keyword_set(silent) eq 0 then print, 'Accessing database, please wait...'

spawn, command, output

;check to see that there is some output

if output[0] eq '' then output='# Either autorization failed or problem connecting to server'

return, output

END
