;+
; @file_comments 
;
;  	put your query in a string:<P>
;
;	IDL> query = 'select * from agebin'<P>
;
;	then run the search:<P>
;
;	IDL> result = query_sql_str(query)<P>
;
;	(a little widget will pop up and ask for your username and login.
;	 At the moment there are no stars for password, but I'm working on it!)<P>
;
;	IDL> print, transpose(result)<P>
;
;	result is a csv formated array of strings, one per line of result, with two comments at the top and a list of other things. Later versions will be able to return a structure, but not just yet!
;	You'll need to login for the first query of each session, but shouldn't after that. If you have the wrong login details and need to replace them, simply type /login as a keyword when you call query_sql:<P>
;
;	IDL> result = query_sql_str(query, /login)<P>
;
;   If you're using the Millenium SQL database, you should use a seperate server. Something like:<P>
;  result = query_sql_str(query, server='http://gavows2.xray.mpe.mpg.de:8080/lgalaxies/DBQueryStream' )<P>
; <P>
;  Another complexity with the Millenium is that sometimes the code guesses the wrong format. You can correct this by supplying a base struct to be replicated using the my_struct=.... keyword
;
;       
; @examples
;	result = query_sql_str('select * from agebin', server='http://gavows2.xray.mpe.mpg.de:8080/lgalaxies/DBQueryStream'
;       
; @param query sql query in the form of a string. The string may be multiline, the program will compress the string to a single line though
;
;       
; @returns A structure with fields equivalent to the titles of the columns returned by the query. Also gives a brief report to the shell about what has gone on.
;
; @keyword _extra allows keywords to be passed to subroutines: <p> 
;      <b>server</b> - Change the server from the default value to something else (this can be changed permanently in sql_query.pro if desired) <P>
;		 <b>login</b>  - Force a new attempt at logging in.
; @keyword my_struct - supply you own struct to avoid problems with incorrect format guessing or provide your own field names
; @keyword quiet - suppresses any messages to the shell, both by IDL and wget.
; @history  Originally created Jan 2006 by Ben Panter. <P>Modifications and improvements suggested by Viv Wild, Manfred Kitzbichler, Eric Hayashi and Jeremy Blaizot. All SQL servers maintained by Gerard Lemson
;       
; @requires An account on the MPA/GAVO webaccess to SQL servers.
;
; @author Ben Panter
;
;
; @version $Id: query_sql_str.pro,v 1.10 2006/07/18 14:55:21 bdp Exp $
;-

FUNCTION query_sql_str, query, my_struct=my_struct, quiet=quiet, _extra=e


;first perform query
if keyword_set(quiet) then csv_result=query_sql(query, /silent, _extra=e) else csv_result=query_sql(query, _extra=e)

;check whether query returned any rows
if n_elements(csv_result) eq 1 then return,'Query Failed'
if keyword_set(quiet) eq 0 then print, 'Database returned: '+strmid(csv_result[0],1)

datastart = (where(strmid(csv_result, 0,1) ne '#'))(0)

if datastart eq -1 then return, ''
if datastart+1 eq n_elements(csv_result) then return, ''

;fieldnames = STRSPLIT(csv_result[datastart], ',', /extract)
;sample_data = strsplit(csv_result[datastart+1], ',', /extract)

n_records = n_elements(csv_result)-datastart - 1l ; JB added "-1l"
hd=csv_result[0:datastart] 
hds=hd[where(strmid(hd,0,7) eq '#COLUMN')]
n_fields=n_elements(hds)

if keyword_set(my_struct) then begin

	if keyword_set(quiet) eq 0 then print, 'using my_struct rather than attempting to recover from header'
	result = replicate(my_struct,n_records)

endif else begin

	var_name=strarr(n_fields)
	var_type=strarr(n_fields)

	;split into parameter names and datatypes
	for i=0, n_fields-1 do begin
		var_name[i]='"'+strmid(hds[i], strpos(hds[i], 'name=')+5,$
		   strpos(hds[i],'JDBC_TYPE=')-strpos(hds[i], 'name=')-6) + '"'
		var_type[i]=strmid(hds[i], strpos(hds[i],'JDBC_TYPE=')+10,$
		   strpos(hds[i],'JDBC_TYPENAME=')-strpos(hds[i],'JDBC_TYPE=')-11)
		end

	;now convert the datatypes into something that IDL can deal with

	var_type_idl=strarr(n_fields)

	bit_loc=where(var_type eq '-7')
	int_loc=where(var_type eq '-6' or var_type eq '5')
	long_loc=where(var_type eq '4')
	long64_loc=where(var_type eq '-5')
	float_loc=where(var_type eq '6')
	double_loc=where(var_type eq '7' or var_type eq '8')
	string_loc=where(var_type eq '1' or var_type eq '12' or var_type eq '-1' or var_type eq '91' or var_type eq '92' or var_type eq '93' or var_type eq '0')
	binary_loc=where(var_type eq '-2' or var_type eq '-3' or var_type eq '-4') 

	if bit_loc[0] ne -1 then var_type_idl[bit_loc]='0b'
	if int_loc[0] ne -1 then var_type_idl[int_loc]='0'
	if long_loc[0] ne -1 then var_type_idl[long_loc]='0L'
	if long64_loc[0] ne -1 then var_type_idl[long64_loc]='long64(0)'
	if float_loc[0] ne -1 then var_type_idl[float_loc]='0.0'
	if double_loc[0] ne -1 then var_type_idl[double_loc]='0.0d'
	if string_loc[0] ne -1 then var_type_idl[string_loc]="'zero'"
	if binary_loc[0] ne -1 then var_type_idl[binary_loc]='0b'

	;name_and_type = [[var_name], [var_type], [var_type_idl]]

	;name_and_type=query_header(csv_result[0:datastart])

	big_line_str=strjoin(strjoin(transpose([[var_name],[var_type_idl]]), ', '), ', ')

	comm = 'res=create_struct('+big_line_str+')'
	ro   = execute(comm)
	result = replicate(res, n_records)

endelse

if keyword_set(quiet) eq 0 then print,'Parsing a search returning ' + string(n_records) + ' lines'

for i=1l, n_records do begin        
    vals=strsplit(csv_result[datastart+i], ',', /extract, /preserve_null)
    for j=0,n_fields-1 do result[i-1].(j) = vals[j]     
endfor

if keyword_set(quiet) eq 0 then print,'n_records = '+strtrim(n_records,2)

return, result
	
END



